/******************************************************************************
*                BC759x演示程序 标准MCS-51 版
*              (使用键盘驱动库和LED显示驱动库)
*
* 程序用途：BC759x LED显示及键盘接口芯片演示，可直接用于BC7591开发板，如果
*           用于BC7595芯片，因该芯片显示只有6位，部分显示超出了显示范围，
*           需要修改显示的位置
* CPU: 标准8051/8052
* RTOS: 无
* Main Frequency: 11.0592MHz
* 版权所有: 北京凌志比高科技有限公司 https://bitcode.com.cn
* 版本号: V1.0
* 初始版本：2020年9月25日
* 版本历史:
*   2020年9月   V1.0 使用led_disp_lib驱动库V3.0 API
* 功能说明: 在DIG4-DIG6提供3位十进制数字显示，并在DIG0-DIG1提供1个16段光柱显示，(DIG0-DIG1)
*           不使用数码管而使用光柱，参见BC7591开发板电路图)。使用3个按键，分别为
*		    "+"(1键) ， "-"(0键) ， "SHIFT"(3键)。数字为闪烁显示
*		    1. 单独按 +/- 键，数字加一或减一
*		    2. 长按 +/- 键3秒，停止闪烁，数字快速增加减小，释放按键后恢复闪烁
*		    3. 同时按SHIFT和+/-键，调整光柱
*		    4. 如果30秒无按键，显示亮度调暗，进入节能状态，按任意键后恢复
* 
* 使用环境： 本程序使用SDCC编译器调试通过。
* 程序说明：
*	  程序使用定时器和串口两个中断。
*     驱动库位于 /key_scan/ 和 /led_disp/ 子目录中。关于驱动库的使用更多信息请查询
*    《UART单线键盘接口驱动库技术手册》及《UART单线LED显示接口驱动库技术手册》。
*	  本示例设置驱动库工作于检测按键释放模式，是为了长按快进模式时能检测按键的释放。但
*	  程序并未对不同按键的释放做分别处理。
******************************************************************************/
#include "8052.h" /* 标准51寄存器定义 */
#include "key_scan/key_scan.h"
#include "led_disp/led_disp.h"
#include "stc8g.h" /* STC8G单片机扩展寄存器定义 */

/*       BC759x连接于UART         */
/*   P3.0 (MCU RX) --> BC759x TX  */
/*   P3.1 (MCU TX) --> BC759x RX  */

#define TIMER0_RELOAD (65536 - 54250) /* 11.0592MHz下设置T0中断时间为50ms（大约） */

void chip_init(void);
void io_init(void);
void uart_init(void);
void timer_init(void);
void interrupt_init(void);

/* 供驱动库调用的UART操作函数 */
void write_uart(uint8_t);
void enable_uart_int(void);
void disable_uart_int(void);

void delay_50ms(uint8_t T);
void display_bar16(uint16_t BarVal, uint8_t Pos); /* 在Pos位显示16段光柱 */

/* 组合键定义 */
const uint8_t  cb1[]    = { 2, 126, 0, 3 }; /* 组合键1 由2个键-键0和键3组成( shift_- )，定义键值为126 */
const uint8_t  cb2[]    = { 2, 125, 1, 3 }; /* 组合键2 由2个键-键1和键3组成( shift_+ )，定义键值为125 */
const uint8_t* CbList[] = { cb1, cb2 };     /* 组合键列表，定义2个组合键 */
uint8_t        CBMap[2];                    /* 检测组合键所需的映射数组，数组大小等于组合键数量 */

/* 长按键定义 */
const uint8_t  lp1[]    = { 0, 124 };          /* 长按键1 定义为0号键(-)长按，定义键值为124 */
const uint8_t  lp2[]    = { 1, 123 };          /* 长按键2 定义为1号键(+)长按，定义键值为123 */
const uint8_t  nokey[]  = { 255, 255 };        /* 长按键3 定义为无按键(用于做长时间无按键检测) */
const uint8_t* LpList[] = { lp1, lp2, nokey }; /* 长按键列表，定义3个长按键 */

volatile uint16_t DownCounter;    /* 延时计时器 */
uint8_t           Value = 100;    /* 显示的数值 */
uint16_t          Bar1  = 0x0001; /* 显示的光柱值 */

int main(void)
{
    uint8_t NoKeyTimer = 0;
    uint8_t NewKey;

    /* 键盘驱动库设置 */
    set_detect_mode(1);                 /* 检测按键释放 */
    set_longpress_count(60);            /* 设置长按键时间为3秒 */
    def_combined_key(CbList, CBMap, 2); /* 设置组合键 */
    def_longpress_key(LpList, 3);       /* 设置长按键和无按键检测 */

    /* 显示驱动库设置 */
    set_write_func(write_uart);      /* 设置写UART函数 */
    set_eni_func(enable_uart_int);   /* 设置使能UART TX中断函数 */
    set_disi_func(disable_uart_int); /* 设置禁止UART TX中断函数 */

    chip_init();
    io_init();
    uart_init();
    timer_init();
    interrupt_init();

    delay_50ms(6);             /* 延时300ms，确保BC759X进入工作状态 */
    clear();                   /* 清除显示和闪烁 */
    send_cmd(BLINK_SPEED, 20); /* 设置闪烁速度 */
    digit_blink(4, 1);         /* 设置第4,5,6显示位为闪烁显示 */
    digit_blink(5, 1);
    digit_blink(6, 1);
    display_dec(Value, 6, 3); /* 在第4,5,6位上显示数值 */
                              /* 此依BC7591开发板电路设计，如果电路为小编号数码管在右，
                                 请在 disp_config.h 中设置
                                 #define LOW_DIG_NUM_ON_RIGHT 1
                                 并将语句改为  display_dec(Value, 4, 3);  */
    display_bar16(Bar1, 0);   /* 显示光柱 */

    while (1)
    {
        /****************************/
        /* 其它任务处理可以放在这里 */
        /****************************/

        /************ 按键处理 ***********/
        if (is_key_changed()) /* 如果有新按键变化 */
        {
            NewKey = get_key_value();
            switch (NewKey)
            {
            case 1: /* 如果是“+”键按下 */
                if (Value < 255)
                {
                    display_dec(++Value, 6, 3);
                }
                break;
            case 123:              /* "+"键长按 */
                digit_blink(4, 0); /* 停止闪烁显示 */
                digit_blink(5, 0);
                digit_blink(6, 0);
                while (get_key_value() == 123) /* 如果是长按键一直持续，Value快速增加 */
                {
                    if (Value < 255)
                    {
                        display_dec(++Value, 6, 3); /* 数值加1,然后刷新数值显示 */
                    }
                    delay_50ms(4); /* 延时200ms */
                }
                digit_blink(4, 1); /* 按键释放，恢复闪烁显示 */
                digit_blink(5, 1);
                digit_blink(6, 1);
                break;
            case 0: /* “-”键按下 */
                if (Value > 0)
                {
                    display_dec(--Value, 6, 3);
                }
                break;
            case 124:              /* "-"键长按 */
                digit_blink(4, 0); /* 停止闪烁显示 */
                digit_blink(5, 0);
                digit_blink(6, 0);
                while (get_key_value() == 124) /* 如果是长按键一直持续，Value快速减小 */
                {
                    if (Value > 0)
                    {
                        display_dec(--Value, 6, 3); /* 刷新数值显示，然后数值减1 */
                    }
                    delay_50ms(4); /* 延时200ms */
                }
                digit_blink(4, 1); /* 按键释放，恢复闪烁显示 */
                digit_blink(5, 1);
                digit_blink(6, 1);
                break;
            case 125: /* 如果是 shift_+ 组合键按下 (光柱1增加) */
                Bar1 = (Bar1 << 1) | 0x01;
                display_bar16(Bar1, 0);
                break;
            case 126: /* 如果是 shift_- 组合键按下 (光柱1减小) */
                Bar1 = Bar1 >> 1;
                display_bar16(Bar1, 0);
                break;
            case 0xFF: /* "无按键"的长按键，收到此键值表示距最后一次键盘事件超过了一个长按键时间没有任何键盘操作 */
                NoKeyTimer++;
                if (NoKeyTimer >= 10) /* 如果超过10倍长按键时间 */
                {
                    NoKeyTimer = 10;         /* 维持NoKeyTimer为10 */
                    send_cmd(DIM_CTL, 0x0C); /* 显示亮度调为0x0C(进入节能状态) */
                    /* sleep(); */
                    /* 如果MCU能够被UART中断唤醒，此处可令MCU进入睡眠 */
                }
                break;
            default:
                break;
            }
            if (NewKey != 0xFF) /* 对除了"无按键"以外的任何键盘事件 */
            {
                NoKeyTimer = 0;
                send_cmd(DIM_CTL, 0); /* 亮度调到最高(从节能状态恢复) */
            }
        }
        /*********** 按键处理部分结束 ***********/
    }
}

/* 芯片初始化，所有STC8G单片机相关的较标准51增加的设置内容均在此初始化 */
void chip_init(void)
{
    /*	M1:M0 settings:
		0:0  traditional mode
		0:1  hi current mode
		1:0  hi-impedence
		1:1  open drain  */

    /* 设置I/O口 模式，此处均设为传统51模式 */
    P0M0 = 0;
    P0M1 = 0;
    P1M0 = 0;
    P1M1 = 0;
    P2M0 = 0;
    P2M1 = 0;
    P3M0 = 0;
    P3M1 = 0;
    P4M0 = 0;
    P4M1 = 0;
    P5M0 = 0;
    P5M1 = 0;

    P_SW2  = 0x80; /* CKSEL & CLKDIV 处于XDATA地址空间，使用MOVX A,@DPTR访问前需要先设置P_SW2的bit7为1  */
    CKSEL  = 0x00; /* 使用内部高速RC (烧录时设置内部高速RC振荡频率为 22.1184MHz) */
    CLKDIV = 2;    /* SYSCLK = MCLK/2 得到11.0592MHz */
    P_SW2  = 0x00; /* 回复 P_SW2 */
    AUXR   = 0;    /* 设置定时器0，1，2采用12T模式以使程序和经典51保持兼容 */
}

/* 设置 UART 所用I/O口 */
void io_init(void)
{
    RXD = 1; /* RX口为输入 */
    TXD = 1; /* TX口设为高 */
}

/* 设置 UART 工作模式和波特率 */
void uart_init(void)
{
    PCON = 0;    /* 串口波特率不加倍，不使用帧检测 */
    SCON = 0x50; /* 串口模式1: 8个数据位，1个停止位 */
    TH1  = 0xFD; /* 设置波特率为9600@11.0592MHz */
    TR1  = 1;    /* 启动TIMER1 */
}

/* 设置定时器 */
void timer_init(void)
{
    TMOD = 0x21; /* TIMER0工作于MODE1（16位定时器），TIMER1工作于MODE 2 （自动重载） */
    TH0  = TIMER0_RELOAD / 0x100;
    TL0  = TIMER0_RELOAD & 0xFF;
    TR0  = 1;
}

/* 中断初始化函数 */
void interrupt_init(void)
{
    ES  = 1; /* 使能串口中断 */
    ET0 = 1; /* 使能T0中断 */
    EA  = 1; /* 使能全局中断 */
}

void disable_uart_int(void)
{
    ES = 0; /* 禁止UART TX中断 */
}

void enable_uart_int(void)
{
    ES = 1; /* 使能UART TX中断 */
}

void write_uart(uint8_t Data)
{
    SBUF = Data;
}

void delay_50ms(uint8_t T)
{
    DownCounter = T;
    while (0 != DownCounter)
        ;
}

/* 在Pos位显示16段光柱 */
void display_bar16(uint16_t Bar, uint8_t Pos)
{
    Pos &= 0x1f;                          /* 确保Pos值在有效范围内 */
    send_cmd(DIRECT_WT | Pos, Bar >> 8);  /* 写光柱高8段 */
    send_cmd(DIRECT_WT | (Pos + 1), Bar); /* 写光柱低8段 */
}

/* 定时器溢出中断处理 (通用延时和长按键定时) */
void TIMER0_ISR(void) __interrupt(1) /* TIMER0 中断程序 */
{
    TH0 = TIMER0_RELOAD / 0x100; /* 重载计数器初始值 */
    TL0 = TIMER0_RELOAD & 0xFF;
    if (0 != DownCounter) /* 如果 DownCounter 不为0，每50ms减一 */
    {
        --DownCounter;
    }
    long_press_tick(); /* 呼叫键盘驱动库长按键计数函数 */
}

/* UART 接收中断处理 (按键处理) */
void UART_ISR(void) __interrupt(4)
{
    if (RI) /* 如果是接收中断 （接收到键盘数据） */
    {
        RI = 0;
        update_key_status(SBUF); /* 呼叫键盘驱动库键盘状态更新函数 */
    }
    if (TI) /* 发送中断 */
    {
        TI = 0;     /* 清除中断标志 */
        tx_ready(); /* 通知驱动库可发送下一个数据 */
    }
}
