/******************************************************************************
*			BC759x演示程序 STM32 版 (使用键盘驱动库和LED显示驱动库)
*
* 程序用途：BC759x LED显示及键盘接口芯片演示，可直接用于BC7591开发板，如果
*		用于BC7595芯片，因该芯片显示只有6位，部分显示超出了显示范围，
*		需要修改显示的位置
* CPU: STM32F103C8
* RTOS:	无
* Main Frequency: 72MHz
* 版权所有:	北京凌志比高科技有限公司 https://bitcode.com.cn
* 版本号: V4.0
* 初始版本：2020年9月25日
* 版本历史:
*	2020年9月	V1.0
*   2021年3月    V2.0 键盘部分改为使用驱动库
*   2021年4月    V3.0 LED显示部分改为使用驱动库，修改格式以兼容早期C89标准
*   2021年4月    V4.0 更新为UART显示驱动库V3.0的API
* 功能说明：	在DIG4-DIG6提供3位十进制数字显示，并在DIG0-DIG1提供1个16段光柱显示，(DIG0-DIG1
*           不使用数码管而使用光柱，参见BC7591开发板电路图)。使用3个按键，分别为
*		    "+"(1键) ， "-"(0键) ， "SHIFT"(3键)。数字为闪烁显示
*		    1. 单独按 +/- 键，数字加一或减一
*		    2. 长按 +/- 键3秒，停止闪烁，数字快速增加减小，释放按键后恢复闪烁
*		    3. 同时按SHIFT和+/-键，调整光柱
*		    4. 如果30秒无按键，显示亮度调暗，进入节能状态，按任意键后恢复
*
* 使用环境：	本程序使用GCC编译器调试通过。
* 程序说明：
*	  程序使用定时器和串口两个中断。
*     驱动库位于 /key_scan/ 和 /led_disp/ 子目录中。关于驱动库的使用更多信息请查询
*    《UART单线键盘接口驱动库技术手册》及《UART单线LED显示接口驱动库技术手册》。
*	  本示例设置驱动库工作于检测按键释放模式，是为了长按快进模式时能检测按键的释放。但
*	  程序并未对不同按键的释放做分别处理。
******************************************************************************/

/*********************************  重要!!!  ***********************************
*  请检查LED显示驱动库配置 disp_config.h 中的设置项是否与项目相符                      *
******************************************************************************/
#include "key_scan/key_scan.h"
#include "led_disp/led_disp.h"
#include "stm32f10x.h"

/*	 BC759x连接于USART1  				*/
/*	 PA9  (MCU TX) --> BC759x RX	*/
/*	 PA10 (MCU RX) --> BC759x TX	*/

void rcc_init(void);
void io_init(void);
void uart_init(void);
void timer_init(void);
void interrupt_init(void);

/* 供驱动库调用的UART操作函数 */
void write_uart(uint8_t);
void enable_uart_int(void);
void disable_uart_int(void);

void delay_1ms(uint16_t Ms);
void display_bar16(uint16_t BarVal, uint8_t Pos); /* 在Pos位显示16段光柱 */

/* 组合键定义 */
const uint8_t  cb1[]    = { 2, 126, 0, 3 }; /* 组合键1 由2个键-键0和键3组成( shift_- )，定义键值为126 */
const uint8_t  cb2[]    = { 2, 125, 1, 3 }; /* 组合键2 由2个键-键1和键3组成( shift_+ )，定义键值为125 */
const uint8_t* CbList[] = { cb1, cb2 };     /* 组合键列表，定义2个组合键 */
uint8_t        CBMap[2];                    /* 检测组合键所需的映射数组，数组大小等于组合键数量 */

/* 长按键定义 */
const uint8_t  lp1[]    = { 0, 124 };          /* 长按键1 定义为0号键(-)长按，定义键值为124 */
const uint8_t  lp2[]    = { 1, 123 };          /* 长按键2 定义为1号键(+)长按，定义键值为123 */
const uint8_t  nokey[]  = { 255, 255 };        /* 长按键3 定义为无按键(用于做长时间无按键检测) */
const uint8_t* LpList[] = { lp1, lp2, nokey }; /* 长按键列表，定义3个长按键 */

volatile uint16_t DownCounter;    /*	延时计时器 */
uint8_t           Value = 100;    /* 显示的数值 */
uint16_t          Bar1  = 0x0001; /*	显示的光柱值 */

int main(void)
{
    uint8_t NoKeyTimer = 0;
    uint8_t NewKey;

    /* 键盘驱动库设置 */
    set_detect_mode(1);                 /* 检测按键释放 */
    set_longpress_count(3000);          /* 设置长按键时间为3秒 */
    def_combined_key(CbList, CBMap, 2); /* 设置组合键 */
    def_longpress_key(LpList, 3);       /* 设置长按键和无按键检测 */

    /* 显示驱动库设置 */
    set_write_func(write_uart);       /* 设置写UART函数 */
    set_eni_func(enable_uart_int);   /* 设置使能UART TX中断函数 */
    set_disi_func(disable_uart_int); /* 设置禁止UART TX中断函数 */

    rcc_init();
    io_init();
    interrupt_init();
    uart_init();
    timer_init();

    delay_1ms(300);            /* 延时300ms，确保BC759X进入工作状态 */
    clear();                   /* 清除显示和闪烁 */
    send_cmd(BLINK_SPEED, 20); /* 设置闪烁速度 */
    digit_blink(4, 1);         /* 设置第4,5,6显示位为闪烁显示 */
    digit_blink(5, 1);
    digit_blink(6, 1);
    display_dec(Value, 6, 3); /* 在第4,5,6位上显示数值 */
                              /* 此依BC7591开发板电路设计，如果电路为小编号数码管在右，
                                 请在 disp_config.h 中设置
                                 #define LOW_DIG_NUM_ON_RIGHT 1
                                 并将语句改为  display_dec(Value, 4, 3);  */
    display_bar16(Bar1, 0);   /* 显示光柱 */

    while (1)
    {
        /**************************/
        // 其它任务处理可以放在这里
        /***************************/

        /************ 按键处理 ***********/
        if (is_key_changed()) /* 如果有新按键变化 */
        {
            NewKey = get_key_value();
            switch (NewKey)
            {
            case 1: /* 如果是“+”键按下 */
                if (Value < 255)
                {
                    display_dec(++Value, 6, 3);
                }
                break;
            case 123:              /* "+"键长按 */
                digit_blink(4, 0); /* 停止闪烁显示 */
                digit_blink(5, 0);
                digit_blink(6, 0);
                while (get_key_value() == 123) /* 如果是长按键一直持续，Value快速增加 */
                {
                    if (Value < 255)
                    {
                        display_dec(++Value, 6, 3); /* 数值加1,然后刷新数值显示 */
                    }
                    delay_1ms(200); /* 延时200ms */
                }
                digit_blink(4, 1); /* 按键释放，恢复闪烁显示 */
                digit_blink(5, 1);
                digit_blink(6, 1);
                break;
            case 0: /* “-”键按下 */
                if (Value > 0)
                {
                    display_dec(--Value, 6, 3);
                }
                break;
            case 124:              /* "-"键长按 */
                digit_blink(4, 0); /* 停止闪烁显示 */
                digit_blink(5, 0);
                digit_blink(6, 0);
                while (get_key_value() == 124) /* 如果是长按键一直持续，Value快速减小 */
                {
                    if (Value > 0)
                    {
                        display_dec(--Value, 6, 3); /* 刷新数值显示，然后数值减1 */
                    }
                    delay_1ms(200); /* 延时200ms */
                }
                digit_blink(4, 1); /* 按键释放，恢复闪烁显示 */
                digit_blink(5, 1);
                digit_blink(6, 1);
                break;
            case 125: /* 如果是 shift_+ 组合键按下 (光柱1增加) */
                Bar1 = (Bar1 << 1) | 0x01;
                display_bar16(Bar1, 0);
                break;
            case 126: /* 如果是 shift_- 组合键按下 (光柱1减小) */
                Bar1 = Bar1 >> 1;
                display_bar16(Bar1, 0);
                break;
            case 0xFF: /* "无按键"的长按键，收到此键值表示距最后一次键盘事件超过了一个长按键时间没有任何键盘操作 */
                NoKeyTimer++;
                if (NoKeyTimer >= 10) /* 如果超过10倍长按键时间 */
                {
                    NoKeyTimer = 10;         /* 维持NoKeyTimer为10 */
                    send_cmd(DIM_CTL, 0x0C); /* 显示亮度调为0x0C(进入节能状态) */
                    // sleep();		/* 如果MCU能够被UART中断唤醒，此处可令MCU进入睡眠 */
                }
                break;
            default:
                break;
            }
            if (NewKey != 0xFF) /* 对除了"无按键"以外的任何键盘事件 */
            {
                NoKeyTimer = 0;
                send_cmd(DIM_CTL, 0); /* 亮度调到最高(从节能状态恢复) */
            }
        }
        /*********** 按键处理部分结束 ***********/
    }
}

void rcc_init(void)
{
    /* 使能 AFIO,	IOPA, USART1 */
    RCC->APB2ENR |= RCC_APB2ENR_AFIOEN | RCC_APB2ENR_IOPAEN | RCC_APB2ENR_USART1EN;
}

/* 初始化GPIO函数 function to initialize GPIOs */
void io_init(void)
{
    /* CRL/CRH mostly used values: 
     4 -- float input	(default)
     3 -- push-pull 50MHz
     2 -- push-pull 2MHz
     1 -- push-pull 10MHz
     B -- alternate push-pull	50MHz
     A -- alternate push-pull	2MHz
     9 -- alternate push-pull	10MHz
     8 -- input with pull-up/pull-down
     0 -- analog input
    */

    GPIOA->CRL = 0x44444444; /* PA0-7 保持上电默认状态(悬空输入) */
    GPIOA->CRH = 0x444448A4; /* PA10(USART1 RX) 设置为带上拉/下拉输入,	PA9(UART1 TX) 设为输出 */
    GPIOA->ODR = 0x00000400; /* 设RX(PA10)为内部上拉 */
}

/* 初始化USART1 function to initialize USART1 */
void uart_init(void)
{
    USART1->BRR = 0x1D4C; /*	APB2时钟频率72MHz情况下，设置USART1波特率为9600 */
    /* USART1配置为：接收使能，接收中断使能，发送使能，8个数据位，1个停止位，无奇偶校验 */
    USART1->CR2 = 0;
    USART1->CR3 = 0;
    USART1->CR1 = USART_CR1_UE | USART_CR1_RXNEIE | USART_CR1_TCIE | USART_CR1_RE | USART_CR1_TE;
}

/* 初始化NVIC函数 function to initialize NVIC */
void interrupt_init(void)
{
    NVIC_EnableIRQ(USART1_IRQn); /* 使能USART1中断 enable USART1	interrupt */
}

void timer_init(void)
{
    SysTick_Config(SystemCoreClock / 1000); /* 设置系统滴答定时器周期为1ms */
}

void disable_uart_int(void)
{
	/* 禁止全部串口中断 */
    NVIC_DisableIRQ(USART1_IRQn); /* 使能USART1中断 enable USART1	interrupt */
    
    /* 或者使用下面语句单独禁止发送中断 */
    /* USART1->CR1 &= ~USART_CR1_TCIE; */
}

void enable_uart_int(void)
{
	/* 使能串口中断 */
    NVIC_EnableIRQ(USART1_IRQn); /* 使能USART1中断 enable USART1	interrupt */
    
    /* 或者使用下面语句单独重新使能发送中断 */
    /* USART1->CR1 |= USART_CR1_TCIE; */
}

void write_uart(uint8_t Data)
{
    USART1->DR = Data;
}

/* 通用延时程序 */
void delay_1ms(uint16_t Ms)
{
    DownCounter = Ms + 1;
    while (DownCounter != 0)
        ;
}

/* 在Pos位显示16段光柱 */
void display_bar16(uint16_t Bar, uint8_t Pos)
{
    Pos &= 0x1f;                          /* 确保Pos值在有效范围内 */
    send_cmd(DIRECT_WT | Pos, Bar >> 8);  /* 写光柱高8段 */
    send_cmd(DIRECT_WT | (Pos + 1), Bar); /* 写光柱低8段 */
}

/* USART1中断服务程序	ISR for	USART1 */
/********************************************************************************************
* 当键盘状态有变化时，UART接收到数据产生此中断							  *
* Data will be received	at UART	then generate interrupt	when the keyboard status is changed *
*********************************************************************************************/
void USART1_IRQHandler(void)
{
    if (USART1->SR & USART_SR_RXNE)    /* 如果是接收RXNE中断	 if the	interrupt source is RXNE */
    {
        update_key_status(USART1->DR & 0XFF);    /* 呼叫键盘驱动库键盘状态更新函数，读取DR寄存器同时会清除中断标志 */
    }
    if (USART1->SR & USART_SR_TC)    /* 如果是发送完成TC中断 */
    {
        USART1->SR &= ~USART_SR_TC;   /* 清除中断标志 */
        tx_ready();		/* 通知驱动库可以发送下一个数据 */
    }
}

void SysTick_Handler(void) /* 系统滴答定时器中断服务程序 */
{
    if (DownCounter != 0) /* 如果计数器不为零，则减一	(通用延时用) */
    {
        --DownCounter;
    }
    long_press_tick(); /* 呼叫键盘驱动库长按键计数函数 */
}
